 # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # #
#  This file creates a suggested solution to "Problem Set 2"    #
#  of the course Microeconometrics, taught at SDU, spring 2012  #
 # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # #

## NOTE: before running the program you need to change the directory in R to the folder containing the dataset

# Reading libraries into memory
	library(sem)
	library(car)
	library(xtable)
	library(lmtest)

# Loading dataset NEW7080.csv into memory [this may take a while due to 247199 observations]
	data <- read.table("NEW7080.csv", header=TRUE, sep=",", na.string=".")

	# attaches variable names to dataset, allowing for calling variables individually 
	attach(data)

	# Visual inspection of dataset
	fix(data)

# 1, 2, 3: Discussion in class

# 4: Replicating column 1 of table IV of Angrist & Krueger 1991
	iv.column1 <- lm(LWKLYWGE ~ EDUC+YR20+YR21+YR22+YR23+YR24+YR25+YR26+YR27+YR28)

	# Displaying model estimation
	summary(iv.column1) 


# 5: Discussion in class

# 6: First stage estimation
	first.stage <- lm(EDUC ~ QTR1+QTR2+QTR3+YR20+YR21+YR22+YR23+YR24+YR25+YR26+YR27+YR28)
	summary(first.stage)

	# Predicted educational level for use in manual 2SLS
	pred.educ <- predict.lm(first.stage)

	second.stage <- lm(LWKLYWGE ~ pred.educ+YR20+YR21+YR22+YR23+YR24+YR25+YR26+YR27+YR28)
	summary(second.stage)

# 7: Discussion in class
	# F-test of joint significance of instruments
	linearHypothesis(first.stage, c("QTR1=0", "QTR2=0", "QTR3=0"))

# 8: 2 stage least squares using quarter of birth as instruments
	educ.iv <- tsls(LWKLYWGE ~ EDUC+YR20+YR21+YR22+YR23+YR24+YR25+YR26+YR27+YR28, ~QTR1+QTR2+QTR3+YR20+YR21+YR22+YR23+YR24+YR25+YR26+YR27+YR28)
	summary(educ.iv)

# 10: Replication of column 2 from table IV:
	# creating set of explanatory variables
	iv.column2 <- tsls(LWKLYWGE ~ EDUC+YR20+YR21+YR22+YR23+YR24+YR25+YR26+YR27+YR28,
				 ~ YR20+YR21+YR22+YR23+YR24+YR25+YR26+YR27+YR28+
				   QTR120+QTR121+QTR122+QTR123+QTR124+QTR125+QTR126+QTR127+QTR128+QTR129+
				   QTR220+QTR221+QTR222+QTR223+QTR224+QTR225+QTR226+QTR227+QTR228+QTR229+
				   QTR320+QTR321+QTR322+QTR323+QTR324+QTR325+QTR326+QTR327+QTR328+QTR329)
	summary(iv.column2)



## 11 (EXTRA):

	# Column 3 from table IV
	iv.column3 <- lm (LWKLYWGE ~ EDUC+YR20+YR21+YR22+YR23+YR24+YR25+YR26+YR27+YR28+AGEQ+AGEQSQ)
	summary(iv.column3)

	# Column 4 from table IV
	iv.column4 <-tsls(LWKLYWGE ~ EDUC+YR20+YR21+YR22+YR23+YR24+YR25+YR26+YR27+YR28+AGEQ+AGEQSQ,
				 ~ YR20+YR21+YR22+YR23+YR24+YR25+YR26+YR27+YR28+
				   QTR120+QTR121+QTR122+QTR123+QTR124+QTR125+QTR126+QTR127+QTR128+QTR129+
				   QTR220+QTR221+QTR222+QTR223+QTR224+QTR225+QTR226+QTR227+QTR228+QTR229+
				   QTR320+QTR321+QTR322+QTR323+QTR324+QTR325+QTR326+QTR327+QTR328+QTR329+AGEQ+AGEQSQ)
	summary(iv.column4)

	# Column 5 of table IV
	iv.column5 <- lm (LWKLYWGE ~ EDUC+YR20+YR21+YR22+YR23+YR24+YR25+YR26+YR27+YR28+RACE+MARRIED+SMSA+NEWENG+MIDATL+ENOCENT+WNOCENT+SOATL+ESOCENT+WSOCENT+MT)
	summary(iv.column5)

	# Column 6 of table IV
	iv.column6 <-tsls(LWKLYWGE ~ EDUC+YR20+YR21+YR22+YR23+YR24+YR25+YR26+YR27+YR28+RACE+MARRIED+SMSA+NEWENG+MIDATL+ENOCENT+WNOCENT+SOATL+ESOCENT+WSOCENT+MT,
				 ~ YR20+YR21+YR22+YR23+YR24+YR25+YR26+YR27+YR28+
				   QTR120+QTR121+QTR122+QTR123+QTR124+QTR125+QTR126+QTR127+QTR128+QTR129+
				   QTR220+QTR221+QTR222+QTR223+QTR224+QTR225+QTR226+QTR227+QTR228+QTR229+
				   QTR320+QTR321+QTR322+QTR323+QTR324+QTR325+QTR326+QTR327+QTR328+QTR329+
				   RACE+MARRIED+SMSA+NEWENG+MIDATL+ENOCENT+WNOCENT+SOATL+ESOCENT+WSOCENT+MT)
	summary(iv.column6)

	# Column 7 of table IV
	iv.column7 <- lm (LWKLYWGE ~ EDUC+YR20+YR21+YR22+YR23+YR24+YR25+YR26+YR27+YR28+AGEQ+AGEQSQ+RACE+MARRIED+SMSA+NEWENG+MIDATL+ENOCENT+WNOCENT+SOATL+ESOCENT+WSOCENT+MT)
	summary(iv.column7)	

	# Column 8 of table IV
	iv.column8 <-tsls(LWKLYWGE ~ EDUC+YR20+YR21+YR22+YR23+YR24+YR25+YR26+YR27+YR28+AGEQ+AGEQSQ+RACE+MARRIED+SMSA+NEWENG+MIDATL+ENOCENT+WNOCENT+SOATL+ESOCENT+WSOCENT+MT,
				 ~ YR20+YR21+YR22+YR23+YR24+YR25+YR26+YR27+YR28+AGEQ+AGEQSQ+
				   QTR120+QTR121+QTR122+QTR123+QTR124+QTR125+QTR126+QTR127+QTR128+QTR129+
				   QTR220+QTR221+QTR222+QTR223+QTR224+QTR225+QTR226+QTR227+QTR228+QTR229+
				   QTR320+QTR321+QTR322+QTR323+QTR324+QTR325+QTR326+QTR327+QTR328+QTR329+
				   RACE+MARRIED+SMSA+NEWENG+MIDATL+ENOCENT+WNOCENT+SOATL+ESOCENT+WSOCENT+MT)
	summary(iv.column8)

