 # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # #
#  This file creates a suggested solution to "Problem Set 3"    #
#  of the course Microeconometrics, taught at SDU, spring 2011  #
 # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # #

## NOTE: To drop a variable from the dataframe you can use the "remove()" command
## NOTE: before running the program you need to change the directory in R to the folder containing the dataset

# Reading libraries into memory
	library(lmtest)
	library(sandwich)
	library(car)
	library(nlme)

# Loading dataset grogger.txt into memory
	data <- read.table("grogger.txt", header=TRUE, sep="", na.string=".")

	# attaches variable names to dataset, allowing for calling variables individually 
	attach(data)

# 1: Indicator of being arrested
	arr86 <- as.numeric(narr86>0)


# 2: LPM of probability of being arrested
	arr86.lpm <- lm(arr86~pcnv+tottime+ptime86+avgsen+inc86+black+hispan+born60, x=T, y=T)
	summary(arr86.lpm)
	# Coefficients of LPM
	lpm.coef <- coef(arr86.lpm)
	# Generating vector of explanatory variables
	X <- arr86.lpm$x
	Y <- arr86.lpm$y


# 3: Investigation of model for heteroskedasticity	
	# Visual inspection
	plot(arr86.lpm, which=1)
	
	# Breusch-Pagan test for heteroskedasticity
	bptest(arr86.lpm)
	
# 4: LPM with heteroskedastic robust standard errors
	coeftest(arr86.lpm, vcov=vcovHC(arr86.lpm, type="HC0"))

# 5: Effect on probability of arrest of pcnv going from 0.25 to 0.75
	0.5*lpm.coef[c("pcnv")]

# 6: Joint significance of avgsen and tottime:
	# Non-robust test
	linearHypothesis(arr86.lpm, c("avgsen=0", "tottime=0"))
	# Robust test
	linearHypothesis(arr86.lpm, c("avgsen=0", "tottime=0"), vcov.=vcovHC(arr86.lpm, type="HC0"))

# 7: Estimation of model using logit
	arr86.logit <- glm(arr86~X-1, family=binomial("logit"))
	summary(arr86.logit)
	# Saving coefficients
	beta.logit <- coef(arr86.logit)

# 8: Estimation of model using probit
	arr86.probit <- glm(arr86~-1+X, family=binomial("probit"))
	summary(arr86.probit)
	# Saving coefficients
	beta.probit <- coef(arr86.probit)
	# Saving Xbeta:
	Xb.probit <- arr86.probit$linear.predictor
	
# 9: Comparing logit and probit estimates
	1.6*beta.probit
	beta.logit

# 10: Marginal effects of probit model
	# Mean of explanatory variables
	X.mean <- apply(X, 2, mean)
	# Replacing mean of indicators with either '0' or '1'
	# (we are interested in the marginal effect of a black person born in 1960)
	X.mean[7]<-1
	X.mean[8]<-0
	X.mean[9]<-1

	* The marginal effect of pcnv for the 'average person'
	pcnv.mfx <- dnorm(X.mean%*%beta.probit,0,1)*beta.probit[2]
	cat(" The marginal effect of the average person with the probit model: ", pcnv.mfx, "\n")

	* The marginal effect of pcnv going from 0.25 to 0.75
	X.mean25 <- X.mean
	X.mean25[2] <- 0.25
	X.mean75 <- X.mean
	X.mean75[2] <- 0.75

	Phi.25 <- pnorm(X.mean25%*%beta.probit,0,1)
	Phi.75 <- pnorm(X.mean75%*%beta.probit,0,1)
	
	Phi.75-Phi.25
	
# 11: Average Partial Effect of pcnv
	pcnv.ape <- mean(dnorm(Xb.probit, 0, 1)*beta.probit[2])
	pcnv.ape

# 12: LPM, Marginal effect at mean (MFX) and Average partial effect (APE)
	cat("The effect of pcnv on the probability of being arrested: \n",
	"LPM \t \t MFX \t \t APE \n",
	round(lpm.coef[2],4), "\t", round(pcnv.mfx,4), "\t", round(pcnv.ape,4), "\n")

# 13: Predictive power of the LPM and probit models
	p.arr86.lpm    <- ifelse(predict.lm(arr86.lpm)>0.5, 1, 0)
	p.arr86.probit <- ifelse(fitted(arr86.probit)>0.5, 1, 0)
	
	# Alternatively: 	p.arr86.probit <- ifelse(Xb.probit>0, 1, 0)
	correct.lpm    <- as.numeric(p.arr86.lpm==arr86)
	correct.probit <- as.numeric(p.arr86.probit==arr86)
	
	cat("Fraction of correctly predicted arrests: \n",
	"LPM \t \t Probit \n",
	round(mean(correct.lpm),4), "\t", round(mean(correct.probit),4), "\n")
	
# 14: Adding squared terms to probit model
	arr86.probit2 <- glm(arr86~-1+X+pcnvsq+pt86sq+inc86sq, family=binomial("probit"))	
	summary(arr86.probit2)
	probit2.coef <- arr86.probit2$coef

	# Joint significance of new variables (Wald-test)
	linearHypothesis(arr86.probit2, c("pcnvsq=0", "pt86sq=0", "inc86sq=0"))

	# Joint significance of new variables (LR-test)
	Lur <- logLik(arr86.probit2)
	Lr  <- logLik(arr86.probit)
	LR <- 2*(Lur[1]-Lr[1])
	1-pchisq(LR,3)
	

	# Plotting relationship between pcnv and probability of arrest
	xaxis <- seq(0, 1, length=100)
	yaxis <- probit2.coef[2]*xaxis+probit2.coef[10]*xaxis^2

	plot(xaxis, yaxis, abline(h = c(0), lty=2))

	# When does the probability of conviction have a negative influnce on the probability of arrest
	# ANSWER: Find the extremum point of: 0.2168pcnv-0.8571 pcnv^2=0