 # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # #
#  This file creates a suggested solution to "Problem Set 5"    #
#  of the course Microeconometrics, taught at SDU, fall 2012  #
 # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # #

## NOTE: before running the program you need to change the directory in R to the folder containing the dataset

# Reading libraries into memory
	# 'bptest' command for Breush-Pagan test of heteroskedasticity
	library(lmtest)
	# tobit estimation ('survreg')
	library(survival)
	# Tobit estimation (vglm)
	library(VGAM)


# Loading dataset tobacco.csv into memory
	data <- read.table("tobacco.csv", header=T, sep=",", na.string=".")
	attach(data)

# 1: OLS estimation of alcohol consumption
	# Estimating linear model
	model.ols <- lm(share1~1+age+nadults+nkids+nkids2+lnx+agelnx+nadlnx)
	summary(model.ols)

	# Investigating appropriateness of linear model
	plot(model.ols, which=1)
	bptest(model.ols)


# 2: The problems are due to the corner solution possibility of alcohol consumption
	hist(share1)


# 3: Tobit estimation of alcohol consumption
	# Using VGAM package
	model.tob   <-  vglm(share1~age+nadults+nkids+nkids2+lnx+agelnx+nadlnx, tobit(Lower=0, Upper=1), data=data)
	summary(model.tob)
	
  # Using Survival-package
	model.tobit <- survreg(Surv(share1, share1>0, type='left') ~ 1+age+nadults+nkids+nkids2+lnx+agelnx+nadlnx,
                         dist='gaussian', data=data,  x=T)
	summary(model.tobit)
  coeftest(model.tobit)

	# Extracting beta-coefficients from tobit estimation
	beta.tobit <- coef(model.tobit)
	# Extracting sigma
	sigma.tobit <- model.tobit$scale

	
	# Vector of explanatory variables
	X <- model.tobit$x	


# 4: Marginal Effects from tobit model
	# Estimating p(Y>0|X)
	p = pnorm(X%*%beta.tobit/sigma.tobit)

	# E[Y|X] (Compare to OLS estimates)
		mfx <- p%*%beta.tobit
		# Average marginal effect
	colnames(X)	
  apply(mfx,2,mean)

	
		# Marginal effect of average individual
		X.mean <- apply(X,2,mean)
		p.mean <-pnorm(X.mean%*%beta.tobit/sigma.tobit)
		mfx.mean <- p.mean%*%beta.tobit
	  colnames(X)	
	  mfx.mean

	# E[Y|X,Y>0] (Can be compared to probit-estimates)
	mills <- dnorm(X%*%beta.tobit/sigma.tobit)/pnorm(X%*%beta.tobit/sigma.tobit)
	adj.factor <- 1-mills*(X%*%beta.tobit/sigma.tobit + mills)

	mfx.yGEzero <- adj.factor%*%beta.tobit
	colnames(X)	
	apply(mfx.yGEzero,2,mean)


# 5: Effect of log total expenditure (Note it works through 3 different coefficients)
	# In the following the average marginal effect is estimated
	# E[Y|X]	
	mfx.lnx <- p*(beta.tobit[6]+beta.tobit[7]*age+beta.tobit[8]*nadults)
	mean(mfx.lnx)

	# E[Y|X,Y>0]
	mfx.lnx.yGEzero <- adj.factor*(beta.tobit[6]+beta.tobit[7]*age+beta.tobit[8]*nadults)
	mean(mfx.lnx.yGEzero)

	# Marginal effect at the mean on E[Y|X] 
	p.mean*(beta.tobit[6]+beta.tobit[7]*X.mean[2]+beta.tobit[8]*X.mean[3])


# 6: Including squared log total expenditure
	model.tobit2 <- survreg(Surv(share1, share1>0, type='left') ~ 1+age+age2+nadults+nkids+nkids2+lnx+lnx2+agelnx+nadlnx, data=data, dist='gaussian', x=T)
	summary(model.tobit2)


# 7: Testing relevance of newly included squared term
	# Wald test
	waldtest(model.tobit2, model.tobit)

  
	# Likelihood ratio test
	LR <-	2*(model.tobit2$loglik[2]-model.tobit$loglik[2])
	1-pchisq(LR, df=2)

  
# 8: Probit model on probability of consuming alcohol
	alcohol <- as.numeric(share1>0)
	model.probit <- glm(alcohol~-1+X, family=binomial("probit"))
	summary(model.probit)
	beta.probit <- coef(model.probit)
	
	# Comparing probit coefficients to normalized tobit coefficients
	norm.tobit <- beta.tobit/sigma.tobit
	cat("Comparing estimated coefficients of probit to normalized tobit model: \n",
	"Variable \t Probit \t \t Tobit \t \t \t \t Norm. Tobit \n",
	"intercept", beta.probit[1], "\t", beta.tobit[1], "\t \t", norm.tobit[1], "\n",
	"age", "\t \t \t", beta.probit[2], "\t", beta.tobit[2], "\t \t", norm.tobit[2], "\n",
	"nadults", "\t", beta.probit[3], "\t \t", beta.tobit[3], "\t \t", norm.tobit[3], "\n",		
	"nkids", "\t \t", beta.probit[4], beta.tobit[4], "\t", norm.tobit[4], "\n",
	"nkids2", "\t", beta.probit[5], "\t", beta.tobit[5], "\t", norm.tobit[5], "\n",
	"lnx", "\t \t \t", beta.probit[6], "\t \t", beta.tobit[6], "\t \t", norm.tobit[6], "\n",
	"agelnx", "\t", beta.probit[7], "\t", beta.tobit[7], norm.tobit[7], "\n",
	"nadlnx", "\t", beta.probit[8], "\t", beta.tobit[8], "\t", norm.tobit[8], "\n")


# 9: LM test for heteroskedasticity related to age and number of adults
	# The procedure for testing for heteroskedasticity (Wooldridge page 685-687):
	# 1: Estimate the model under H0 (i.e. assuming homoskedasticity)
	# 2: Find partial derivatives (using 17.21 and 17.22 from Wooldridge)
	# 3: Evaluate these partial derivatives at H0 for each observation
	# 4: Run OLS on a unit vector, using the partial derivatives as explanatory variables,
	#    along with the log likelihood evaluated at the k variables on which heteroskedasticity is a potential problem
	# 5: The LM statistic is then N-SSR or N*R^2 from this regression, under the null, this
	#    is Chi-squared distributed with k degrees of freedom. 


	# step 3A: Estimate partial derivatives of log-likelihood function evaluated at H0 and the parameter estimates
	noshare1 <- as.numeric(share1==0)
		age_i <- -noshare1*dnorm(X%*%beta.tobit/sigma.tobit)/(1-pnorm(X%*%beta.tobit/sigma.tobit))*age/sigma.tobit+
                (1-noshare1)*(share1-X%*%beta.tobit)/sigma.tobit^2*age
		nadults_i <- -noshare1*dnorm(X%*%beta.tobit/sigma.tobit)/(1-pnorm(X%*%beta.tobit/sigma.tobit))*nadults/sigma.tobit+
                (1-noshare1)*(share1-X%*%beta.tobit)/sigma.tobit^2*nadults
		nkids_i 	<- -noshare1*dnorm(X%*%beta.tobit/sigma.tobit)/(1-pnorm(X%*%beta.tobit/sigma.tobit))*nkids/sigma.tobit+
                (1-noshare1)*(share1-X%*%beta.tobit)/sigma.tobit^2*nkids
		nkids2_i 	<- -noshare1*dnorm(X%*%beta.tobit/sigma.tobit)/(1-pnorm(X%*%beta.tobit/sigma.tobit))*nkids2/sigma.tobit	+
                (1-noshare1)*(share1-X%*%beta.tobit)/sigma.tobit^2*nkids2
		lnx_i 	<- -noshare1*dnorm(X%*%beta.tobit/sigma.tobit)/(1-pnorm(X%*%beta.tobit/sigma.tobit))*lnx/sigma.tobit+
                (1-noshare1)*(share1-X%*%beta.tobit)/sigma.tobit^2*lnx
		agelnx_i 	<- -noshare1*dnorm(X%*%beta.tobit/sigma.tobit)/(1-pnorm(X%*%beta.tobit/sigma.tobit))*agelnx/sigma.tobit	+
                (1-noshare1)*(share1-X%*%beta.tobit)/sigma.tobit^2*agelnx
		nadlnx_i 	<- -noshare1*dnorm(X%*%beta.tobit/sigma.tobit)/(1-pnorm(X%*%beta.tobit/sigma.tobit))*nadlnx/sigma.tobit	+
                (1-noshare1)*(share1-X%*%beta.tobit)/sigma.tobit^2*nadlnx

	# Step 3B: Partial derivatives with respect to sigma
		sigma.tobit2 <- sigma.tobit^2
	  sigma.tobit3 <- sigma.tobit^3
	
		sigma_i	 <- (noshare1*dnorm(X%*%beta.tobit/sigma.tobit)*(X%*%beta.tobit))/(2*sigma.tobit3*(1-pnorm(X%*%beta.tobit/sigma.tobit)))+(1-noshare1)*((share1-X%*%beta.tobit)^2/(2*sigma.tobit2^2)-1/(2*sigma.tobit2))

	# Step 3C: Partial derivatives with respect to the two extra parameters (under H1)
		ageextra_i 	<- sigma.tobit2*sigma_i*age
		nadultsextra_i <- sigma.tobit2*sigma_i*nadults

	# Step 4: Regression of created variables on a vector of ones
		n <- length(age_i)
		ones <- c(rep(1,n))
		het.ols <- lm(ones ~ -1+age_i+nadults_i+nkids_i+nkids2_i+lnx_i+agelnx_i+nadlnx_i+sigma_i+ageextra_i+nadultsextra_i)
		summary(het.ols)

	# Step 5: Obtaining test-statistic
		LM <- n*0.02465
		1-pchisq(LM,2)
	