 # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # #
#  This file creates a suggested solution to "Problem Set 6"    #
#  of the course Microeconometrics, taught at SDU, fall 2012  #
 # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # #

## NOTE: before running the program you need to change the directory in R to the folder containing the dataset

# Reading libraries into memory
	# Allowing removing missing observations
	library(agricolae)
	# Enables making robust standard errors ('vcovHC'-command):
	library(sandwich)
	# Allows using the 'coeftest' and 'bptest' commands:
	library(lmtest)
	# Allows using panel data estimation (the 'plm' command):
	library(plm)
	# Allows using the 'linearHypothesis' command
	library(car)

# Loading dataset KEANE.csv into memory
	data <- read.table("KEANE.csv", header=T, sep=",", na.string=".")
	attach(data)

	data.na <- delete.na(data, "greater")
	# The 'data.na' is necessary to compute the clustered standard errors

# 1: The panel data is not balanced. There are 1881 individuals (id), observed between 1981 and 1987
# with a total of 12723 observations. This means there is some attrition.
	apply(data,2,sum)

# 2: Wage regression
	# Suggested explanatory variables: educ expwc expwc^2 expbc expbc^2 expser expser^2 manuf black y82 y83 y84 y85 y86 y87
	# Alternative: educ exper expersq manuf black y82 y83 y84 y85 y86 y87

# 3: Estimating the linear model
	# Estimating linear model
	model.alt <- lm(lwage~1+educ+exper+expersq+manuf+black+y82+y83+y84+y85+y86+y87, x=T, data=data)
	summary(model.alt)
	model.ols <- lm(lwage~1+educ+expwc+I(expwc^2)+expbc+I(expbc^2)+expser+I(expser^2)+manuf+black+y82+y83+y84+y85+y86+y87, x=T, data=data.na)
	summary(model.ols)
	X <- model.ols$x

	# Investigating appropriateness of linear model
	plot(model.ols, which=1)
	bptest(model.ols)

	# I.e. we reject the null-hypothesis of homoskedastic standard errors
	
	# Estimation using Huber-White heteroskedastic standard errors
	coeftest(model.ols, vcov=vcovHC(model.ols, type="HC0"))
	
	# The following makes clustered standard errors	
	clx <-
	function(fm, dfcw, cluster){
	M <- length(unique(cluster))
	N <- length(cluster)
	dfc <- (M/(M-1))*((N-1)/(N-fm$rank))
	u <- apply(estfun(fm),2, function(x) tapply(x, cluster, sum))
	vcovCL <- dfc*sandwich(fm, meat=crossprod(u)/N)*dfcw
	coeftest(fm, vcovCL) }
	
	clx(model.ols, 1, data.na$id)

# 4: Model without year-dummies
	model2.ols <- lm(lwage~1+educ+expwc+I(expwc^2)+expbc+I(expbc^2)+expser+I(expser^2)+manuf+black, data=data)
	summary(model2.ols)
  waldtest(model.ols,model2.ols)
	
# 5: Test of significance of experience variables
	# Wald test
	linearHypothesis(model.ols, c("expwc=0", "expbc=0", "expser=0", "I(expbc^2)=0", "I(expwc^2)=0", "I(expser^2)=0"), vcov.=vcovHC(model.ols, type="HC0"))

	# LR-test
	model.res <- lm(lwage~1+educ+manuf+black+y82+y83+y84+y85+y86+y87, data=data.na)
	lrtest(model.res, model.ols)

  #Manually: (needed if you use plm() to run pooled OLS. Have calculated SSR manually since I have not found an easy way to extract them)
  n<-length(data.na$id) 
  SSR_ur<-sum(residuals(model.ols)^2)
	SSR_r<-sum(residuals(model.res)^2)
	LogLikRatio<-(SSR_ur/SSR_r)^(-n/2)
  TestStat<-2*log(LogLikRatio)
  1-pchisq(TestStat,6)  
  
# 6: Fixed effects estimation (within transformation)
	data.fe <- pdata.frame(data, index = c("id", "year"), row.names = TRUE)
	model.fe <- plm(lwage~1+educ+expwc+I(expwc^2)+expbc+I(expbc^2)+expser+I(expser^2)+manuf+black+y82+y83+y84+y85+y86+y87, data=data.fe, model = "within")
	summary(model.fe)   #Observe: We lose Black because it is time invariant. 
  fe<-fixef(model.fe)  #Extract fixed effects
  pFtest(model.fe,model.ols)  #Significant fixed effects test 
  
# 7: Random Effects Estimation
	model.re <- plm(lwage~1+educ+expwc+I(expwc^2)+expbc+I(expbc^2)+expser+I(expser^2)+manuf+black+y82+y83+y84+y85+y86+y87, data=data.fe, model = "random")
	summary(model.re)
 

# 8: First difference estimation
	model.fd <- plm(lwage~1+educ+expwc+I(expwc^2)+expbc+I(expbc^2)+expser+I(expser^2)+manuf+black+y82+y83+y84+y85+y86+y87, data=data.fe, model = "fd")
	summary(model.fd)


# 9: Discussion in class
	model.re2 <- plm(lwage~1+educ+expwc+I(expwc^2)+expbc+I(expbc^2)+expser+I(expser^2)+manuf, data=data.fe, model = "random")
	model.fe2 <- plm(lwage~1+educ+expwc+I(expwc^2)+expbc+I(expbc^2)+expser+I(expser^2)+manuf+black, data=data.fe, model = "within")
  # Hausman test of RE against FE (null being that the Ci's are independent of the X's)
	phtest(model.fe2, model.re2)

	# Alternative way of making the same Hausman test:
	form <- lwage~1+educ+expwc+I(expwc^2)+expbc+I(expbc^2)+expser+I(expser^2)+manuf
	phtest(form, data=data.fe, model=c("within", "random"))

	# The output tells us that there are significant differences between the RE and FE models, and hence that 
	# we should use the fixed effects estimations. (NOTE however the problems with the FE model 
	# regarding the potential OVB from 'black' being excluded as discussed in class)

	# Testing Random versus Pooled OLS
	# If the above Hausman test had not indicated the presence of correlation between the individual effects and
	# the explanatory variables (and hence the need for FE-estimation) we would have to run a 
	# Breusch-Pagan LM-test for the presence of Random effecst. If no individual specific random effects are
	# present using pooled OLS will be more efficient.
	# H0: var(ci)=0 - i.e. no random effects are needed
	model.pooling <- plm(lwage~1+educ+expwc+I(expwc^2)+expbc+I(expbc^2)+expser+I(expser^2)+manuf+y82+y83+y84+y85+y86+y87, data=data.fe, model = "pooling")
	plmtest(model.pooling, type=c("bp"))
	# Here we would reject the null, that is, the RE-model is more appropriate than the pooled OLS